searchData={"items":[{"type":"behaviour","title":"Cache","doc":"# ElixirCache\n[![Hex version badge](https://img.shields.io/hexpm/v/elixir_cache.svg)](https://hex.pm/packages/elixir_cache)\n[![Test](https://github.com/MikaAK/elixir_cache/actions/workflows/test.yml/badge.svg)](https://github.com/MikaAK/elixir_cache/actions/workflows/test.yml)\n[![Credo](https://github.com/MikaAK/elixir_cache/actions/workflows/credo.yml/badge.svg)](https://github.com/MikaAK/elixir_cache/actions/workflows/credo.yml)\n[![Dialyzer](https://github.com/MikaAK/elixir_cache/actions/workflows/dialyzer.yml/badge.svg)](https://github.com/MikaAK/elixir_cache/actions/workflows/dialyzer.yml)\n[![Coverage](https://github.com/MikaAK/elixir_cache/actions/workflows/coverage.yml/badge.svg)](https://github.com/MikaAK/elixir_cache/actions/workflows/coverage.yml)\n\nThe goal of this project is to unify Cache APIs and make Strategies easy to implement and sharable\nacross all storage types/adapters\n\nThe second goal is to make sure testing of all cache related funciions is easy, meaning caches should be isolated\nper test and not leak their state to outside tests","ref":"Cache.html"},{"type":"behaviour","title":"Installation - Cache","doc":"The package can be installed by adding `elixir_cache` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:elixir_cache, \"~> 0.1.0\"}\n  ]\nend\n```\n\nThe docs can be found at  .","ref":"Cache.html#module-installation"},{"type":"behaviour","title":"Usage - Cache","doc":"```elixir\ndefmodule MyModule do\n  use Cache,\n    adapter: Cache.Redis,\n    name: :my_name,\n    sandbox?: Mix.env() === :test,\n    opts: [...opts]\nend\n```\n\nIn our `application.ex`\n```elixir\nchildren = [\n  {Cache, [MyModule]}\n]\n```\n\nNow we can use `MyModule` to call various Cache apis\n\n```elixir\nMyModule.get(\"key\") #> {:ok, nil}\nMyModule.put(\"key\", \"value\") #> :ok\nMyModule.get(\"key\") #> {:ok, \"value\"}\n```","ref":"Cache.html#module-usage"},{"type":"behaviour","title":"Adapters - Cache","doc":"- `Cache.Agent` - Simple agent based caching\n- `Cache.DETS`  - Disk persisted caching with [dets](https://www.erlang.org/doc/man/dets.html)\n- `Cache.ETS`   - Super quick in-memory cache with [`ets`](https://www.erlang.org/doc/man/ets.html)\n- `Cache.Redis` - Caching adapter using [Redix](https://github.com/whatyouhide/redix) & [Poolboy](https://github.com/devinus/poolboy), supports Redis JSON an Redis Hashes\n- `Cache.ConCache` - Wrapper around [ConCache](https://github.com/sasa1977/con_cache) library\n\n#### Adapter Specific Functions\nSome adapters have specific functions such as redis which has hash functions and pipeline functions to make calls easier.\n\nThese adapter when used will add extra commands to your cache module.","ref":"Cache.html#module-adapters"},{"type":"behaviour","title":"Sandboxing - Cache","doc":"Our cache config accepts a `sandbox?: boolean`. In sandbox mode, the `Cache.Sandbox` adapter will be used, which is just a simple Agent cache unique to the root process. The `Cache.SandboxRegistry` is responsible for registering test processes to a\nunique instance of the Sandbox adapter cache. This makes it safe in test mode to run all your tests asynchronously!\n\nFor test isolation via the `Cache.SandboxRegistry` to work, you must start the registry in your setup, or your test_helper.exs:\n\n```elixir\n# test/test_helper.exs\n\n+ Cache.SandboxRegistry.start_link()\nExUnit.start()\n\n```\n\nThen inside our `setup` for a test we can do:\n\n```elixir\nCache.SandboxRegistry.start([MyCache, CacheItem])\n```","ref":"Cache.html#module-sandboxing"},{"type":"behaviour","title":"Creating Adapters - Cache","doc":"Adapters are very easy to create in this model and are basically just a module that implement the `@behaviour Cache`\n\nThis behaviour adds the following callbacks\n\n```\nput(cache_name, key, ttl, value, opts \\\\ [])\nget(cache_name, key, opts \\\\ [])\ndelete(cache_name, key, opts \\\\ [])\nopts_definition() # NimbleOptions definition map\nchild_spec({cache_name, cache_opts})\n```\n\n`Cache.ETS` is probably the easiest adapter to follow as a guide as it's a simple `Task`","ref":"Cache.html#module-creating-adapters"},{"type":"behaviour","title":"Runtime Configuration - Cache","doc":"Adapter configuration can also be specified at runtime. These options are first passed to the adapter\nchild_spec when starting the adapter and then passed to all runtime function calls.\n\nFor example:\n\n```elixir\n  # Configure with Module Function\n  defmodule Cache.Example do\n    use Cache,\n      adapter: Cache.Redis,\n      name: :test_cache_redis,\n      opts: {Cache.Example, :opts, []}\n\n    def opts, do: [host: \"localhost\", port: 6379]\n  end\n\n  # Configure with callback function\n  defmodule Cache.Example do\n    use Cache,\n      adapter: Cache.Redis,\n      name: :test_cache_redis,\n      opts: &Cache.Example.opts/0\n\n    def opts, do: [host: \"localhost\", port: 6379]\n  end\n\n  # Fetch from application config\n  # config :elixir_cache, Cache.Example, []\n  defmodule Cache.Example do\n    use Cache,\n      adapter: Cache.Redis,\n      name: :test_cache_redis,\n      opts: :elixir_cache\n  end\n\n  # Fetch from application config\n  # config :elixir_cache, :cache_opts, []\n  defmodule Cache.Example do\n    use Cache,\n      adapter: Cache.Redis,\n      name: :test_cache_redis,\n      opts: {:elixir_cache, :cache_opts}\n  end\n```\n\nRuntime options can be configured in one of the following formats:\n\n* `{module, function, args}` - Module, function, args\n* `{application_name, key}` - Application name. This is called as `Application.fetch_env!(application_name, key)`.\n* `application_name` - Application name as an atom. This is called as `Application.fetch_env!(application_name, cache_module})`.\n* `function` - Zero arity callback function. For eg. `&YourModule.options/0`\n* `[key: value_type]` - Keyword list of options.","ref":"Cache.html#module-runtime-configuration"},{"type":"function","title":"Cache.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Cache.html#child_spec/1"},{"type":"callback","title":"Cache.child_spec/1","doc":"","ref":"Cache.html#c:child_spec/1"},{"type":"callback","title":"Cache.delete/2","doc":"","ref":"Cache.html#c:delete/2"},{"type":"callback","title":"Cache.delete/3","doc":"","ref":"Cache.html#c:delete/3"},{"type":"callback","title":"Cache.get/2","doc":"","ref":"Cache.html#c:get/2"},{"type":"callback","title":"Cache.get/3","doc":"","ref":"Cache.html#c:get/3"},{"type":"function","title":"Cache.get_or_create/3","doc":"Retrieves a value from the cache if it exists, or executes a function to create and store it.  \n\nThis is a convenience function implementing the common \"get or create\" pattern for caches. \nIt attempts to fetch a value from the cache first, and only if the value doesn't exist, \nit will execute the provided function to generate the value and store it in the cache.","ref":"Cache.html#get_or_create/3"},{"type":"function","title":"Parameters - Cache.get_or_create/3","doc":"- `cache` - The cache module to use (must implement the Cache behaviour)\n- `key` - The key to look up or create\n- `fnc` - A function that returns `{:ok, value}` or `{:error, reason}`","ref":"Cache.html#get_or_create/3-parameters"},{"type":"function","title":"Returns - Cache.get_or_create/3","doc":"- `{:ok, value}` - The value from cache or newly created value\n- `{:error, reason}` - If an error occurred during retrieval or creation","ref":"Cache.html#get_or_create/3-returns"},{"type":"function","title":"Examples - Cache.get_or_create/3","doc":"```elixir\nCache.get_or_create(MyApp.Cache, \"user:123\", fn ->\n  case UserRepo.get(123) do\n    nil -> {:error, ErrorMessage.not_found(\"User not found\")}\n    user -> {:ok, user}\n  end\nend)\n```","ref":"Cache.html#get_or_create/3-examples"},{"type":"function","title":"Cache.init/1","doc":"","ref":"Cache.html#init/1"},{"type":"callback","title":"Cache.opts_definition/0","doc":"","ref":"Cache.html#c:opts_definition/0"},{"type":"callback","title":"Cache.put/4","doc":"","ref":"Cache.html#c:put/4"},{"type":"callback","title":"Cache.put/5","doc":"","ref":"Cache.html#c:put/5"},{"type":"callback","title":"Cache.start_link/1","doc":"","ref":"Cache.html#c:start_link/1"},{"type":"function","title":"Cache.start_link/2","doc":"","ref":"Cache.html#start_link/2"},{"type":"module","title":"Cache.Agent","doc":"This module is the Agent adapter. Very lightweight use only","ref":"Cache.Agent.html"},{"type":"function","title":"Cache.Agent.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Cache.Agent.html#child_spec/1"},{"type":"module","title":"Cache.ConCache","doc":"ETS Based cache https://github.com/sasa1977/con_cache\n\nTakes the following options:\n\n * `:acquire_lock_timeout` (`t:pos_integer/0`) - The default value is `5000`.\n\n* `:touch_on_read` (`t:boolean/0`) - The default value is `false`.\n\n* `:global_ttl` - The default value is `1800000`.\n\n* `:ttl_check_interval` - The default value is `60000`.\n\n* `:dirty?` (`t:boolean/0`) - Use dirty_put instead of locking to put, enabled by default The default value is `true`.\n\n* `:ets_options` - https://www.erlang.org/doc/man/ets.html#new-2","ref":"Cache.ConCache.html"},{"type":"function","title":"Cache.ConCache.dirty_get_or_store/3","doc":"","ref":"Cache.ConCache.html#dirty_get_or_store/3"},{"type":"type","title":"Cache.ConCache.ets_option/0","doc":"","ref":"Cache.ConCache.html#t:ets_option/0"},{"type":"function","title":"Cache.ConCache.get_or_store/4","doc":"Implements a version of get_or_store that locks locally\nso only one process runs `store_fun` at a time.\n\nAny other processes that miss cache will wait for the first\ncaller to finish `store_fun` then will read the result from cache.","ref":"Cache.ConCache.html#get_or_store/4"},{"type":"type","title":"Cache.ConCache.opts/0","doc":"","ref":"Cache.ConCache.html#t:opts/0"},{"type":"module","title":"Cache.DETS","doc":"DETS adapter so that we can use dets as a cache","ref":"Cache.DETS.html"},{"type":"module","title":"Options - Cache.DETS","doc":"* `:ram_file` (`t:boolean/0`) - Enable RAM File The default value is `false`.\n\n* `:type` - Data type of DETS cache The default value is `:set`.\n\n* `:file_path` (`t:String.t/0`) - File path to save DETS file at The default value is `\"./\"`.","ref":"Cache.DETS.html#module-options"},{"type":"function","title":"Cache.DETS.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\n`arg` is passed as the argument to `Task.start_link/1` in the `:start` field\nof the spec.\n\nFor more information, see the `Supervisor` module,\nthe `Supervisor.child_spec/2` function and the `t:Supervisor.child_spec/0` type.","ref":"Cache.DETS.html#child_spec/1"},{"type":"module","title":"Cache.ETS","doc":"ETS (Erlang Term Storage) adapter for high-performance in-memory caching.\n\nThis adapter provides a fast, process-independent cache using Erlang's built-in ETS tables.\nIt's ideal for applications requiring low-latency access to cached data within a single node.","ref":"Cache.ETS.html"},{"type":"module","title":"Features - Cache.ETS","doc":"* In-memory storage with configurable concurrency options\n* Direct access to ETS-specific operations\n* Very high performance for read and write operations\n* Support for atomic counter operations","ref":"Cache.ETS.html#module-features"},{"type":"module","title":"Options - Cache.ETS","doc":"* `:write_concurrency` (`t:boolean/0`) - Enable write concurrency\n\n* `:read_concurrency` (`t:boolean/0`) - Enable read concurrency\n\n* `:decentralized_counters` (`t:boolean/0`) - Use decentralized counters\n\n* `:type` - Data type of ETS cache The default value is `:set`.\n\n* `:compressed` (`t:boolean/0`) - Enable ets compression","ref":"Cache.ETS.html#module-options"},{"type":"module","title":"Example - Cache.ETS","doc":"```elixir\ndefmodule MyApp.Cache do\n  use Cache,\n    adapter: Cache.ETS,\n    name: :my_app_cache,\n    opts: [\n      read_concurrency: true,\n      write_concurrency: true\n    ]\nend\n```","ref":"Cache.ETS.html#module-example"},{"type":"function","title":"Cache.ETS.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\n`arg` is passed as the argument to `Task.start_link/1` in the `:start` field\nof the spec.\n\nFor more information, see the `Supervisor` module,\nthe `Supervisor.child_spec/2` function and the `t:Supervisor.child_spec/0` type.","ref":"Cache.ETS.html#child_spec/1"},{"type":"module","title":"Cache.Redis","doc":"Redis adapter for distributed caching with ElixirCache.\n\nThis adapter provides a connection pool to Redis, enabling distributed caching\nacross multiple nodes or services. It supports the standard Cache behavior plus\nadditional Redis-specific operations like hash manipulation, JSON operations, and sets.","ref":"Cache.Redis.html"},{"type":"module","title":"Features - Cache.Redis","doc":"* Connection pooling for efficient Redis access\n* Support for Redis URI connection strings\n* Hash operations for storing field-value pairs within a key\n* JSON operations for working with complex nested data structures\n* Set operations for managing collections\n* Direct access to Redis commands via pipeline and command functions","ref":"Cache.Redis.html#module-features"},{"type":"module","title":"Options - Cache.Redis","doc":"* `:uri` (`t:String.t/0`) - Required. The connection uri to redis\n\n* `:size` (`t:pos_integer/0`) - The amount of workers in the pool\n\n* `:max_overflow` (`t:pos_integer/0`) - The amount of max overflow the pool can handle\n\n* `:strategy` - The type of queue to use for poolboy","ref":"Cache.Redis.html#module-options"},{"type":"module","title":"Example - Cache.Redis","doc":"```elixir\ndefmodule MyApp.RedisCache do\n  use Cache,\n    adapter: Cache.Redis,\n    name: :my_app_redis_cache,\n    opts: [\n      uri: \"redis://localhost:6379\",\n      size: 10,\n      max_overflow: 5\n    ]\nend\n```","ref":"Cache.Redis.html#module-example"},{"type":"function","title":"Cache.Redis.command/3","doc":"","ref":"Cache.Redis.html#command/3"},{"type":"function","title":"Cache.Redis.command!/3","doc":"","ref":"Cache.Redis.html#command!/3"},{"type":"function","title":"Cache.Redis.hash_delete/4","doc":"","ref":"Cache.Redis.html#hash_delete/4"},{"type":"function","title":"Cache.Redis.hash_get/4","doc":"","ref":"Cache.Redis.html#hash_get/4"},{"type":"function","title":"Cache.Redis.hash_get_all/3","doc":"","ref":"Cache.Redis.html#hash_get_all/3"},{"type":"function","title":"Cache.Redis.hash_get_many/3","doc":"","ref":"Cache.Redis.html#hash_get_many/3"},{"type":"function","title":"Cache.Redis.hash_scan/4","doc":"","ref":"Cache.Redis.html#hash_scan/4"},{"type":"function","title":"Cache.Redis.hash_set/6","doc":"","ref":"Cache.Redis.html#hash_set/6"},{"type":"function","title":"Cache.Redis.hash_set_many/4","doc":"","ref":"Cache.Redis.html#hash_set_many/4"},{"type":"function","title":"Cache.Redis.hash_values/3","doc":"","ref":"Cache.Redis.html#hash_values/3"},{"type":"function","title":"Cache.Redis.json_array_append/5","doc":"","ref":"Cache.Redis.html#json_array_append/5"},{"type":"function","title":"Cache.Redis.json_clear/4","doc":"","ref":"Cache.Redis.html#json_clear/4"},{"type":"function","title":"Cache.Redis.json_delete/4","doc":"","ref":"Cache.Redis.html#json_delete/4"},{"type":"function","title":"Cache.Redis.json_get/4","doc":"","ref":"Cache.Redis.html#json_get/4"},{"type":"function","title":"Cache.Redis.json_incr/5","doc":"","ref":"Cache.Redis.html#json_incr/5"},{"type":"function","title":"Cache.Redis.json_set/5","doc":"","ref":"Cache.Redis.html#json_set/5"},{"type":"function","title":"Cache.Redis.ms_to_nearest_sec/1","doc":"","ref":"Cache.Redis.html#ms_to_nearest_sec/1"},{"type":"function","title":"Cache.Redis.pipeline/3","doc":"","ref":"Cache.Redis.html#pipeline/3"},{"type":"function","title":"Cache.Redis.pipeline!/3","doc":"","ref":"Cache.Redis.html#pipeline!/3"},{"type":"function","title":"Cache.Redis.pool_config/2","doc":"","ref":"Cache.Redis.html#pool_config/2"},{"type":"function","title":"Cache.Redis.sadd/4","doc":"","ref":"Cache.Redis.html#sadd/4"},{"type":"function","title":"Cache.Redis.scan/3","doc":"","ref":"Cache.Redis.html#scan/3"},{"type":"function","title":"Cache.Redis.smembers/3","doc":"","ref":"Cache.Redis.html#smembers/3"},{"type":"module","title":"Cache.Sandbox","doc":"Sandbox adapter for isolated testing of applications using ElixirCache.\n\nThis module provides a mock implementation of all cache adapters, allowing tests to run in isolation\nwithout interfering with each other's data. The sandbox uses a basic `Agent` to store data in memory\nand implements the full range of caching operations supported by all adapters.","ref":"Cache.Sandbox.html"},{"type":"module","title":"Features - Cache.Sandbox","doc":"* Isolated cache namespaces for concurrent testing\n* Support for all standard cache operations\n* Implementation of Redis-specific features like hash and JSON operations\n* Implementation of ETS-specific operations for complete testing compatibility\n* Lightweight in-memory storage for fast test execution","ref":"Cache.Sandbox.html#module-features"},{"type":"module","title":"Usage - Cache.Sandbox","doc":"The Sandbox adapter is typically enabled through the `sandbox?` option when defining a cache module:\n\n```elixir\ndefmodule MyApp.TestCache do\n  use Cache,\n    adapter: Cache.Redis,  # Original adapter doesn't matter when sandbox is enabled\n    name: :test_cache,\n    opts: [],\n    sandbox?: Mix.env() == :test\nend\n```\n\nIn your tests, use `Cache.SandboxRegistry.start(MyApp.TestCache)` in the setup block to ensure\nproper isolation between test cases.\n\n> **Note**: This adapter should not be used in production environments.","ref":"Cache.Sandbox.html#module-usage"},{"type":"function","title":"Cache.Sandbox.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"Cache.Sandbox.html#child_spec/1"},{"type":"function","title":"Cache.Sandbox.command/3","doc":"","ref":"Cache.Sandbox.html#command/3"},{"type":"function","title":"Cache.Sandbox.command!/3","doc":"","ref":"Cache.Sandbox.html#command!/3"},{"type":"function","title":"Cache.Sandbox.from_dets/2","doc":"","ref":"Cache.Sandbox.html#from_dets/2"},{"type":"function","title":"Cache.Sandbox.from_ets/2","doc":"","ref":"Cache.Sandbox.html#from_ets/2"},{"type":"function","title":"Cache.Sandbox.hash_delete/4","doc":"","ref":"Cache.Sandbox.html#hash_delete/4"},{"type":"function","title":"Cache.Sandbox.hash_get/4","doc":"","ref":"Cache.Sandbox.html#hash_get/4"},{"type":"function","title":"Cache.Sandbox.hash_get_all/3","doc":"","ref":"Cache.Sandbox.html#hash_get_all/3"},{"type":"function","title":"Cache.Sandbox.hash_get_many/3","doc":"","ref":"Cache.Sandbox.html#hash_get_many/3"},{"type":"function","title":"Cache.Sandbox.hash_scan/4","doc":"","ref":"Cache.Sandbox.html#hash_scan/4"},{"type":"function","title":"Cache.Sandbox.hash_set/6","doc":"","ref":"Cache.Sandbox.html#hash_set/6"},{"type":"function","title":"Cache.Sandbox.hash_set_many/4","doc":"","ref":"Cache.Sandbox.html#hash_set_many/4"},{"type":"function","title":"Cache.Sandbox.hash_values/3","doc":"","ref":"Cache.Sandbox.html#hash_values/3"},{"type":"function","title":"Cache.Sandbox.info/1","doc":"","ref":"Cache.Sandbox.html#info/1"},{"type":"function","title":"Cache.Sandbox.info/2","doc":"","ref":"Cache.Sandbox.html#info/2"},{"type":"function","title":"Cache.Sandbox.insert_raw/2","doc":"","ref":"Cache.Sandbox.html#insert_raw/2"},{"type":"function","title":"Cache.Sandbox.json_array_append/5","doc":"","ref":"Cache.Sandbox.html#json_array_append/5"},{"type":"function","title":"Cache.Sandbox.json_clear/4","doc":"","ref":"Cache.Sandbox.html#json_clear/4"},{"type":"function","title":"Cache.Sandbox.json_delete/4","doc":"","ref":"Cache.Sandbox.html#json_delete/4"},{"type":"function","title":"Cache.Sandbox.json_get/4","doc":"","ref":"Cache.Sandbox.html#json_get/4"},{"type":"function","title":"Cache.Sandbox.json_incr/5","doc":"","ref":"Cache.Sandbox.html#json_incr/5"},{"type":"function","title":"Cache.Sandbox.json_set/5","doc":"","ref":"Cache.Sandbox.html#json_set/5"},{"type":"function","title":"Cache.Sandbox.match_delete/2","doc":"","ref":"Cache.Sandbox.html#match_delete/2"},{"type":"function","title":"Cache.Sandbox.match_object/2","doc":"","ref":"Cache.Sandbox.html#match_object/2"},{"type":"function","title":"Cache.Sandbox.match_object/3","doc":"","ref":"Cache.Sandbox.html#match_object/3"},{"type":"function","title":"Cache.Sandbox.member/2","doc":"","ref":"Cache.Sandbox.html#member/2"},{"type":"function","title":"Cache.Sandbox.pipeline/3","doc":"","ref":"Cache.Sandbox.html#pipeline/3"},{"type":"function","title":"Cache.Sandbox.pipeline!/3","doc":"","ref":"Cache.Sandbox.html#pipeline!/3"},{"type":"function","title":"Cache.Sandbox.sadd/4","doc":"","ref":"Cache.Sandbox.html#sadd/4"},{"type":"function","title":"Cache.Sandbox.scan/3","doc":"","ref":"Cache.Sandbox.html#scan/3"},{"type":"function","title":"Cache.Sandbox.select/2","doc":"","ref":"Cache.Sandbox.html#select/2"},{"type":"function","title":"Cache.Sandbox.select/3","doc":"","ref":"Cache.Sandbox.html#select/3"},{"type":"function","title":"Cache.Sandbox.select_delete/2","doc":"","ref":"Cache.Sandbox.html#select_delete/2"},{"type":"function","title":"Cache.Sandbox.smembers/3","doc":"","ref":"Cache.Sandbox.html#smembers/3"},{"type":"function","title":"Cache.Sandbox.to_dets/2","doc":"","ref":"Cache.Sandbox.html#to_dets/2"},{"type":"function","title":"Cache.Sandbox.to_ets/1","doc":"","ref":"Cache.Sandbox.html#to_ets/1"},{"type":"function","title":"Cache.Sandbox.update_counter/3","doc":"","ref":"Cache.Sandbox.html#update_counter/3"},{"type":"module","title":"Cache.SandboxRegistry","doc":"This module is used to start the sandbox registry and register caches in test mode\n\nMore details soon...","ref":"Cache.SandboxRegistry.html"},{"type":"function","title":"Cache.SandboxRegistry.find!/2","doc":"","ref":"Cache.SandboxRegistry.html#find!/2"},{"type":"function","title":"Cache.SandboxRegistry.register_caches/2","doc":"","ref":"Cache.SandboxRegistry.html#register_caches/2"},{"type":"function","title":"Cache.SandboxRegistry.start/1","doc":"","ref":"Cache.SandboxRegistry.html#start/1"},{"type":"function","title":"Cache.SandboxRegistry.start_link/0","doc":"","ref":"Cache.SandboxRegistry.html#start_link/0"},{"type":"module","title":"Cache.Metrics","doc":"Add the following metrics for elixir_cache:\n\nMetrics included:\n  - `elixir_cache.cache.get.count`\n  - `elixir_cache.cache.get.miss.count`\n  - `elixir_cache.cache.get.error.count`\n  - `elixir_cache.cache.put.count`\n  - `elixir_cache.cache.delete.count`\n  - `elixir_cache.cache.get.duration.millisecond`","ref":"Cache.Metrics.html"},{"type":"function","title":"Cache.Metrics.metrics/0","doc":"","ref":"Cache.Metrics.html#metrics/0"},{"type":"extras","title":"Introduction to ElixirCache","doc":"# Introduction to ElixirCache\n\nElixirCache is a standardized and testable caching library for Elixir applications. It provides a consistent interface for various cache backends, allowing you to seamlessly switch between different storage mechanisms while maintaining the same API.","ref":"introduction.html"},{"type":"extras","title":"Key Features - Introduction to ElixirCache","doc":"- **Multiple Cache Adapters**: Support for ETS, DETS, Redis, Agent, and ConCache\n- **Standardized Interface**: Common API for all adapters\n- **Test Isolation**: Sandbox adapter for isolated testing\n- **Telemetry Integration**: Built-in metrics and telemetry events\n- **Feature-rich**: Support for TTL, JSON operations, and more advanced features","ref":"introduction.html#key-features"},{"type":"extras","title":"When to Use ElixirCache - Introduction to ElixirCache","doc":"ElixirCache is ideal for:\n\n- Caching frequently accessed data to reduce database load\n- Temporary storage of computation results\n- Session storage\n- Implementing distributed caching across multiple nodes\n- Applications requiring different caching strategies in different environments","ref":"introduction.html#when-to-use-elixircache"},{"type":"extras","title":"Getting Started - Introduction to ElixirCache","doc":"Check out our [installation guide](tutorials/installation.md) to begin using ElixirCache in your projects.","ref":"introduction.html#getting-started"},{"type":"extras","title":"Installation","doc":"# Installation\n\nThis guide will walk you through the process of installing ElixirCache and setting up your first cache.","ref":"installation.html"},{"type":"extras","title":"Adding ElixirCache to Your Project - Installation","doc":"Add `:elixir_cache` to your list of dependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:elixir_cache, \"~> 0.3.8\"}\n  ]\nend\n```\n\nThen run `mix deps.get` to fetch the dependency.","ref":"installation.html#adding-elixircache-to-your-project"},{"type":"extras","title":"Initial Setup - Installation","doc":"","ref":"installation.html#initial-setup"},{"type":"extras","title":"Basic Setup with ETS Cache - Installation","doc":"To set up a basic ETS cache, create a module that uses the Cache macro:\n\n```elixir\ndefmodule MyApp.Cache do\n  use Cache,\n    adapter: Cache.ETS,\n    name: :my_app_cache\nend\n```\n\nThen add the cache to your application's supervision tree in `application.ex`:\n\n```elixir\ndef start(_type, _args) do\n  children = [\n    # ... other children\n    MyApp.Cache\n  ]\n\n  opts = [strategy: :one_for_one, name: MyApp.Supervisor]\n  Supervisor.start_link(children, opts)\nend\n```","ref":"installation.html#basic-setup-with-ets-cache"},{"type":"extras","title":"Verifying Your Installation - Installation","doc":"To verify your cache is working, you can try storing and retrieving a value:\n\n```elixir\n# Store a value\nMyApp.Cache.put(\"my_key\", \"Hello, ElixirCache!\")\n\n# Retrieve a value\n{:ok, value} = MyApp.Cache.get(\"my_key\")\nIO.puts(value) # Outputs: Hello, ElixirCache!\n```","ref":"installation.html#verifying-your-installation"},{"type":"extras","title":"Next Steps - Installation","doc":"Once you have ElixirCache installed, you can:\n\n- Learn about [basic cache operations](basic_operations.md)\n- Explore [different adapter options](../how-to/choosing_adapter.md)\n- Set up [Redis caching](../how-to/redis_setup.md) for distributed applications","ref":"installation.html#next-steps"},{"type":"extras","title":"Basic Cache Operations","doc":"# Basic Cache Operations\n\nThis tutorial demonstrates the fundamental operations you can perform with ElixirCache.","ref":"basic_operations.html"},{"type":"extras","title":"Setting Up a Cache Module - Basic Cache Operations","doc":"Before we begin with operations, make sure you have a cache module set up:\n\n```elixir\ndefmodule MyApp.Cache do\n  use Cache,\n    adapter: Cache.ETS,\n    name: :my_app_cache\nend\n```","ref":"basic_operations.html#setting-up-a-cache-module"},{"type":"extras","title":"Basic Operations - Basic Cache Operations","doc":"","ref":"basic_operations.html#basic-operations"},{"type":"extras","title":"Storing Values - Basic Cache Operations","doc":"To store a value in the cache:\n\n```elixir\n# Basic storage with no TTL (Time To Live)\nMyApp.Cache.put(\"user:1\", %{name: \"John\", role: \"admin\"})\n\n# With a TTL of 60 seconds\nMyApp.Cache.put(\"session:123\", %{user_id: 1, authenticated: true}, 60)\n```","ref":"basic_operations.html#storing-values"},{"type":"extras","title":"Retrieving Values - Basic Cache Operations","doc":"To retrieve a value from the cache:\n\n```elixir\ncase MyApp.Cache.get(\"user:1\") do\n  {:ok, user} ->\n    IO.puts(\"Found user: #{user.name}\")\n    \n  {:ok, nil} ->\n    IO.puts(\"User not found in cache\")\n    \n  {:error, reason} ->\n    IO.puts(\"Error retrieving from cache: #{reason}\")\nend\n```","ref":"basic_operations.html#retrieving-values"},{"type":"extras","title":"Deleting Values - Basic Cache Operations","doc":"To remove a value from the cache:\n\n```elixir\nMyApp.Cache.delete(\"user:1\")\n```","ref":"basic_operations.html#deleting-values"},{"type":"extras","title":"Get or Create Pattern - Basic Cache Operations","doc":"A common pattern is to check if a value exists in the cache, and if not, create it:\n\n```elixir\n{:ok, user} = MyApp.Cache.get_or_create(\"user:1\", fn ->\n  # This function will only run if the key doesn't exist in the cache\n  {:ok, %{name: \"John\", role: \"admin\"}}\nend)\n```","ref":"basic_operations.html#get-or-create-pattern"},{"type":"extras","title":"Working with Complex Data - Basic Cache Operations","doc":"ElixirCache automatically handles serialization and deserialization of complex Elixir data structures, including:\n\n- Maps\n- Structs\n- Lists\n- Tuples\n- Custom types\n\nYou can store these types without any additional configuration:\n\n```elixir\n# Storing a complex nested structure\ndata = %{\n  users: [\n    %{id: 1, name: \"Alice\", roles: [:admin, :editor]},\n    %{id: 2, name: \"Bob\", roles: [:viewer]}\n  ],\n  settings: %{\n    notifications: true,\n    theme: \"dark\"\n  }\n}\n\nMyApp.Cache.put(\"app:state\", data)\n\n# Later retrieve it with the same structure\n{:ok, retrieved_data} = MyApp.Cache.get(\"app:state\")\n```","ref":"basic_operations.html#working-with-complex-data"},{"type":"extras","title":"Next Steps - Basic Cache Operations","doc":"Now that you understand the basic operations, you might want to explore:\n\n- [Advanced cache techniques](advanced_techniques.md)\n- [Working with different adapters](../how-to/choosing_adapter.md)\n- [Using Redis with ElixirCache](../how-to/redis_setup.md)","ref":"basic_operations.html#next-steps"},{"type":"extras","title":"Advanced Cache Techniques","doc":"# Advanced Cache Techniques\n\nThis tutorial covers more advanced patterns and techniques for using ElixirCache in your applications.","ref":"advanced_techniques.html"},{"type":"extras","title":"Working with Cache TTL (Time To Live) - Advanced Cache Techniques","doc":"Setting appropriate TTL values is important for efficient cache usage:\n\n```elixir\n# Cache for 10 seconds\nMyApp.Cache.put(\"short-lived-key\", :timer.seconds(10), %{data: \"expires quickly\"})\n\n# Cache for 1 hour\nMyApp.Cache.put(\"hourly-key\", :timer.hours(1), %{data: \"expires in an hour\"})\n\n# Cache for 1 day\nMyApp.Cache.put(\"daily-key\", :timer.hours(24), %{data: \"expires in a day\"})\n\n# Cache indefinitely (no TTL)\nMyApp.Cache.put(\"permanent-key\", %{data: \"never expires\"})\n```","ref":"advanced_techniques.html#working-with-cache-ttl-time-to-live"},{"type":"extras","title":"Using Function Memoization - Advanced Cache Techniques","doc":"You can implement function memoization using ElixirCache:\n\n```elixir\ndefmodule MyApp.MemoizedCalculator do\n  def factorial(n, cache \\\\ MyApp.Cache) do\n    cache_key = \"factorial:#{n}\"\n    \n    case cache.get(cache_key) do\n      {:ok, result} when not is_nil(result) ->\n        result\n        \n      _ ->\n        # Calculate the result if not in cache\n        result = calculate_factorial(n)\n        \n        # Store in cache for future use (cache for 1 hour)\n        cache.put(cache_key, 3600, result)\n        \n        result\n    end\n  end\n  \n  defp calculate_factorial(0), do: 1\n  defp calculate_factorial(1), do: 1\n  defp calculate_factorial(n) when n > 1, do: n * factorial(n - 1)\nend\n```","ref":"advanced_techniques.html#using-function-memoization"},{"type":"extras","title":"Implementing Cache Stampede Protection - Advanced Cache Techniques","doc":"Cache stampede occurs when many requests try to regenerate a cached item simultaneously:\n\n```elixir\ndefmodule MyApp.StampedeProtection do\n  # Stale-while-revalidate pattern\n  def get_with_protection(key, ttl, stale_ttl, generator_fn) do\n    case MyApp.Cache.get(key) do\n      # Cache hit - return value\n      {:ok, %{value: value, timestamp: ts}} ->\n        now = System.system_time(:second)\n        \n        # If stale but not expired, return stale value and trigger background refresh\n        if now - ts > stale_ttl and now - ts   \n            refresh_cache(key, ttl, generator_fn) \n          end)\n        end\n        \n        {:ok, value}\n        \n      # Cache miss - generate and store\n      _ ->\n        refresh_cache(key, ttl, generator_fn)\n    end\n  end\n  \n  defp refresh_cache(key, ttl, generator_fn) do\n    with {:ok, value} <- generator_fn.() do\n      cached_value = %{\n        value: value,\n        timestamp: System.system_time(:millisecond)\n      }\n      \n      MyApp.Cache.put(key, ttl, cached_value)\n      {:ok, value}\n    end\n  end\nend\n```","ref":"advanced_techniques.html#implementing-cache-stampede-protection"},{"type":"extras","title":"Using Cache Prefixing for Namespaces - Advanced Cache Techniques","doc":"Organize your cache keys using prefixes:\n\n```elixir\ndefmodule MyApp.UserCache do\n  @prefix \"user:\"\n  \n  def store_user(user_id, user_data) do\n    MyApp.Cache.put(\"#{@prefix}#{user_id}\", user_data)\n  end\n  \n  def get_user(user_id) do\n    MyApp.Cache.get(\"#{@prefix}#{user_id}\")\n  end\n  \n  def invalidate_user(user_id) do\n    MyApp.Cache.delete(\"#{@prefix}#{user_id}\")\n  end\n  \n  # Bulk invalidation in ETS cache\n  def invalidate_all_users() do\n    if MyApp.Cache.cache_adapter() == Cache.ETS do\n      MyApp.Cache.match_delete({:\"#{@prefix}_\", :_})\n    else\n      # Fallback for other adapters without pattern matching\n      {:error, :not_supported}\n    end\n  end\nend\n```","ref":"advanced_techniques.html#using-cache-prefixing-for-namespaces"},{"type":"extras","title":"Caching Database Queries - Advanced Cache Techniques","doc":"A common pattern is caching database query results:\n\n```elixir\ndefmodule MyApp.PostRepository do\n  def get_featured_posts do\n    cache_key = \"featured_posts\"\n    \n    case MyApp.Cache.get(cache_key) do\n      {:ok, posts} when is_list(posts) ->\n        {:ok, posts}\n        \n      _ ->\n        # Query from database if not in cache\n        with {:ok, posts} <- fetch_featured_posts_from_db() do\n          # Cache for 15 minutes\n          :ok = MyApp.Cache.put(cache_key, :timer.minutes(15), posts)\n          {:ok, posts}\n        end\n    end\n  end\n  \n  defp fetch_featured_posts_from_db do\n    # Database query logic here\n    # ...\n  end\nend\n```","ref":"advanced_techniques.html#caching-database-queries"},{"type":"extras","title":"Working with Telemetry for Cache Monitoring - Advanced Cache Techniques","doc":"Implement telemetry handlers to monitor cache performance:\n\n```elixir\ndefmodule MyApp.CacheMonitor do\n  def setup do\n    :telemetry.attach(\n      \"cache-hit-rate-tracker\",\n      [:elixir_cache, :cache, :get],\n      &handle_get_event/4,\n      %{hits: 0, total: 0}\n    )\n    \n    :telemetry.attach(\n      \"cache-miss-tracker\",\n      [:elixir_cache, :cache, :get, :miss],\n      &handle_miss_event/4,\n      %{misses: 0}\n    )\n  end\n  \n  def handle_get_event(_event, _measurements, _metadata, state) do\n    total = state.total + 1\n    \n    if rem(total, 100) == 0 do\n      hit_rate = state.hits / total\n      IO.puts(\"Cache hit rate: #{hit_rate * 100}%\")\n    end\n    \n    %{state | total: total}\n  end\n  \n  def handle_miss_event(_event, _measurements, _metadata, state) do\n    misses = state.misses + 1\n    %{state | misses: misses}\n  end\nend\n```","ref":"advanced_techniques.html#working-with-telemetry-for-cache-monitoring"},{"type":"extras","title":"How to Choose the Right Cache Adapter","doc":"# How to Choose the Right Cache Adapter\n\nElixirCache supports multiple cache adapters, each with its own strengths and use cases. This guide will help you choose the most appropriate adapter for your specific needs.","ref":"choosing_adapter.html"},{"type":"extras","title":"Available Adapters - How to Choose the Right Cache Adapter","doc":"ElixirCache provides the following adapters:\n\n1. `Cache.ETS` - Erlang Term Storage\n2. `Cache.DETS` - Disk-based ETS\n3. `Cache.Redis` - Redis-backed distributed cache\n4. `Cache.Agent` - Simple Agent-based in-memory cache\n5. `Cache.ConCache` - ConCache wrapper\n6. `Cache.Sandbox` - Isolated cache for testing","ref":"choosing_adapter.html#available-adapters"},{"type":"extras","title":"Choosing an Adapter - How to Choose the Right Cache Adapter","doc":"","ref":"choosing_adapter.html#choosing-an-adapter"},{"type":"extras","title":"Cache.ETS - How to Choose the Right Cache Adapter","doc":"**Best for:**\n- High-performance in-memory caching\n- Single node applications\n- Low-latency requirements\n\n**Configuration example:**\n\n```elixir\ndefmodule MyApp.Cache do\n  use Cache,\n    adapter: Cache.ETS,\n    name: :my_app_cache,\n    opts: [\n      read_concurrency: true,\n      write_concurrency: true\n    ]\nend\n```","ref":"choosing_adapter.html#cache-ets"},{"type":"extras","title":"Cache.DETS - How to Choose the Right Cache Adapter","doc":"**Best for:**\n- Persistent caching across application restarts\n- Larger datasets that shouldn't be lost on restart\n- Less frequent access patterns\n\n**Configuration example:**\n\n```elixir\ndefmodule MyApp.PersistentCache do\n  use Cache,\n    adapter: Cache.DETS,\n    name: :my_app_persistent_cache,\n    opts: [\n      file: \"cache_data.dets\"\n    ]\nend\n```","ref":"choosing_adapter.html#cache-dets"},{"type":"extras","title":"Cache.Redis - How to Choose the Right Cache Adapter","doc":"**Best for:**\n- Distributed applications running on multiple nodes\n- Systems requiring shared cache across services\n- Applications needing advanced features like expiration, pub/sub, etc.\n\n**Configuration example:**\n\n```elixir\ndefmodule MyApp.DistributedCache do\n  use Cache,\n    adapter: Cache.Redis,\n    name: :my_app_redis_cache,\n    opts: [\n      host: \"localhost\",\n      port: 6379,\n      pool_size: 5\n    ]\nend\n```","ref":"choosing_adapter.html#cache-redis"},{"type":"extras","title":"Cache.Agent - How to Choose the Right Cache Adapter","doc":"**Best for:**\n- Simple use cases\n- Small applications\n- Development environments\n\n**Configuration example:**\n\n```elixir\ndefmodule MyApp.SimpleCache do\n  use Cache,\n    adapter: Cache.Agent,\n    name: :my_app_simple_cache\nend\n```","ref":"choosing_adapter.html#cache-agent"},{"type":"extras","title":"Cache.ConCache - How to Choose the Right Cache Adapter","doc":"**Best for:**\n- Applications already using ConCache\n- Needs for automatic key expiration and callback execution\n\n**Configuration example:**\n\n```elixir\ndefmodule MyApp.ConCache do\n  use Cache,\n    adapter: Cache.ConCache,\n    name: :my_app_con_cache,\n    opts: [\n      ttl_check_interval: :timer.seconds(1),\n      global_ttl: :timer.minutes(10)\n    ]\nend\n```","ref":"choosing_adapter.html#cache-concache"},{"type":"extras","title":"Cache.Sandbox - How to Choose the Right Cache Adapter","doc":"**Best for:**\n- Testing environments\n- Isolated tests that shouldn't interfere with each other\n\n**Configuration example:**\n\n```elixir\ndefmodule MyApp.TestCache do\n  use Cache,\n    adapter: Cache.ETS,\n    name: :my_app_test_cache,\n    sandbox?: true\nend\n```","ref":"choosing_adapter.html#cache-sandbox"},{"type":"extras","title":"Switching Between Adapters - How to Choose the Right Cache Adapter","doc":"One of the main benefits of ElixirCache is the ability to easily switch between adapters without changing your application code. You can use different adapters in different environments:\n\n```elixir\ndefmodule MyApp.Cache do\n  use Cache,\n    adapter: get_adapter(),\n    name: :my_app_cache,\n    opts: get_opts()\n\n  defp get_adapter do\n    case Mix.env() do\n      :test -> Cache.Sandbox\n      :dev -> Cache.ETS\n      :prod -> Cache.Redis\n    end\n  end\n\n  defp get_opts do\n    case Mix.env() do\n      :test -> []\n      :dev -> [read_concurrency: true]\n      :prod -> [\n        host: System.get_env(\"REDIS_HOST\", \"localhost\"),\n        port: String.to_integer(System.get_env(\"REDIS_PORT\", \"6379\")),\n        pool_size: 10\n      ]\n    end\n  end\nend\n```","ref":"choosing_adapter.html#switching-between-adapters"},{"type":"extras","title":"Performance Considerations - How to Choose the Right Cache Adapter","doc":"When choosing an adapter, consider:\n\n1. **Access patterns** - How frequently are you reading vs writing?\n2. **Data volume** - How much data will be stored?\n3. **Persistence requirements** - Does the data need to survive restarts?\n4. **Distribution needs** - Will multiple nodes/services need access?\n5. **Complexity** - Do you need advanced features or simple key-value storage?\n\nAlways benchmark different options with your specific workload to determine the best fit.","ref":"choosing_adapter.html#performance-considerations"},{"type":"extras","title":"How to Set Up Redis Caching","doc":"# How to Set Up Redis Caching\n\nThis guide walks you through configuring ElixirCache with Redis as the backend.","ref":"redis_setup.html"},{"type":"extras","title":"Prerequisites - How to Set Up Redis Caching","doc":"Before setting up Redis with ElixirCache, make sure you have:\n\n1. Redis server installed and running\n2. ElixirCache properly installed in your project","ref":"redis_setup.html#prerequisites"},{"type":"extras","title":"Basic Redis Configuration - How to Set Up Redis Caching","doc":"","ref":"redis_setup.html#basic-redis-configuration"},{"type":"extras","title":"Step 1: Define Your Redis Cache Module - How to Set Up Redis Caching","doc":"Create a module that uses the Cache functionality with the Redis adapter:\n\n```elixir\ndefmodule MyApp.RedisCache do\n  use Cache,\n    adapter: Cache.Redis,\n    name: :my_app_redis,\n    opts: [\n      host: \"localhost\",\n      port: 6379,\n      pool_size: 5\n    ]\nend\n```","ref":"redis_setup.html#step-1-define-your-redis-cache-module"},{"type":"extras","title":"Step 2: Add to Your Supervision Tree - How to Set Up Redis Caching","doc":"Make sure to add your Redis cache to your application's supervision tree:\n\n```elixir\ndef start(_type, _args) do\n  children = [\n    # ... other children\n    MyApp.RedisCache\n  ]\n\n  opts = [strategy: :one_for_one, name: MyApp.Supervisor]\n  Supervisor.start_link(children, opts)\nend\n```","ref":"redis_setup.html#step-2-add-to-your-supervision-tree"},{"type":"extras","title":"Redis Configuration Options - How to Set Up Redis Caching","doc":"The Redis adapter supports various configuration options:\n\n```elixir\ndefmodule MyApp.RedisCache do\n  use Cache,\n    adapter: Cache.Redis,\n    name: :my_app_redis,\n    opts: [\n      # Connection settings\n      host: \"redis.example.com\",\n      port: 6379,\n      password: \"your_password\",  # Optional\n      database: 0,                # Optional, default is 0\n      \n      # Connection pool settings\n      pool_size: 10,              # Number of connections in the pool\n      max_overflow: 5,            # Maximum number of overflow workers\n      \n      # Timeout settings\n      timeout: 5000,              # Connection timeout in milliseconds\n      \n      # SSL options\n      ssl: true,                  # Enable SSL\n      ssl_opts: [                 # SSL options\n        verify: :verify_peer,\n        cacertfile: \"/path/to/ca_certificate.pem\",\n        certfile: \"/path/to/client_certificate.pem\",\n        keyfile: \"/path/to/client_key.pem\"\n      ],\n      \n      # Encoding options\n      compression_level: 1        # Level of compression (0-9, higher = more compression)\n    ]\nend\n```","ref":"redis_setup.html#redis-configuration-options"},{"type":"extras","title":"Environment-Based Configuration - How to Set Up Redis Caching","doc":"For better maintainability, consider using environment variables or configuration files:\n\n```elixir\ndefmodule MyApp.RedisCache do\n  use Cache,\n    adapter: Cache.Redis,\n    name: :my_app_redis,\n    opts: [\n      host: System.get_env(\"REDIS_HOST\", \"localhost\"),\n      port: String.to_integer(System.get_env(\"REDIS_PORT\", \"6379\")),\n      password: System.get_env(\"REDIS_PASSWORD\"),\n      pool_size: String.to_integer(System.get_env(\"REDIS_POOL_SIZE\", \"10\"))\n    ]\nend\n```","ref":"redis_setup.html#environment-based-configuration"},{"type":"extras","title":"Working with Redis-Specific Features - How to Set Up Redis Caching","doc":"The Redis adapter provides some Redis-specific functionality beyond the standard cache interface:","ref":"redis_setup.html#working-with-redis-specific-features"},{"type":"extras","title":"Hash Operations - How to Set Up Redis Caching","doc":"```elixir\n# Set hash field\nMyApp.RedisCache.hash_set(\"user:1\", \"name\", \"John\")\nMyApp.RedisCache.hash_set(\"user:1\", \"email\", \"john@example.com\")\n\n# Get hash field\n{:ok, name} = MyApp.RedisCache.hash_get(\"user:1\", \"name\")\n\n# Get all hash fields\n{:ok, user_data} = MyApp.RedisCache.hash_get_all(\"user:1\")\n\n# Delete hash field\nMyApp.RedisCache.hash_delete(\"user:1\", \"email\")\n```","ref":"redis_setup.html#hash-operations"},{"type":"extras","title":"JSON Operations - How to Set Up Redis Caching","doc":"```elixir\n# Store JSON\njson_data = %{users: [%{id: 1, name: \"Alice\"}, %{id: 2, name: \"Bob\"}]}\nMyApp.RedisCache.json_set(\"app:data\", \".\", json_data)\n\n# Get JSON \n{:ok, data} = MyApp.RedisCache.json_get(\"app:data\", \".\")\n\n# Get nested JSON path\n{:ok, users} = MyApp.RedisCache.json_get(\"app:data\", \".users\")\n\n# Update specific path\nMyApp.RedisCache.json_set(\"app:data\", \".users[0].name\", \"Alicia\")\n```","ref":"redis_setup.html#json-operations"},{"type":"extras","title":"Troubleshooting Redis Connection - How to Set Up Redis Caching","doc":"If you're having issues connecting to Redis:\n\n1. **Verify Redis is running**: `redis-cli ping` should return `PONG`\n2. **Check connection settings**: Ensure host, port, and password are correct\n3. **Network issues**: Ensure your application can reach the Redis server (firewalls, network rules)\n4. **Authentication**: Confirm your Redis password is correct if authentication is enabled\n5. **Connection pool**: Monitor your connection pool usage to ensure you haven't exhausted connections","ref":"redis_setup.html#troubleshooting-redis-connection"},{"type":"extras","title":"Redis in Production - How to Set Up Redis Caching","doc":"For production environments, consider:\n\n1. **Use a dedicated Redis instance** or managed Redis service\n2. **Enable persistence** in Redis if you need data to survive restarts\n3. **Configure appropriate timeouts** based on your application needs\n4. **Monitor Redis performance** using tools like Prometheus and Grafana\n5. **Set up proper SSL** if connecting to Redis over untrusted networks","ref":"redis_setup.html#redis-in-production"},{"type":"extras","title":"How to Test Applications Using ElixirCache","doc":"# How to Test Applications Using ElixirCache\n\nThis guide explains how to effectively test applications that use ElixirCache, focusing on the sandbox functionality.","ref":"testing_with_cache.html"},{"type":"extras","title":"Using the Sandbox Mode - How to Test Applications Using ElixirCache","doc":"ElixirCache provides a sandbox mode specifically designed for testing. This ensures that your tests:\n\n1. Are isolated from each other\n2. Don't leave lingering cache data between test runs\n3. Can run in parallel without conflicts","ref":"testing_with_cache.html#using-the-sandbox-mode"},{"type":"extras","title":"Configuring Your Cache for Testing - How to Test Applications Using ElixirCache","doc":"In your test environment, you can wrap any cache adapter with the sandbox functionality:\n\n```elixir\n# In lib/my_app/cache.ex\ndefmodule MyApp.Cache do\n  use Cache,\n    adapter: get_cache_adapter(),\n    name: :my_app_cache,\n    opts: get_cache_opts(),\n    # Enable sandbox mode in test environment\n    sandbox?: Mix.env() == :test\n\n  defp get_cache_adapter do\n    case Mix.env() do\n      :test -> Cache.ETS\n      :dev -> Cache.ETS\n      :prod -> Cache.Redis\n    end\n  end\n  \n  defp get_cache_opts do\n    case Mix.env() do\n      :test -> []\n      :dev -> []\n      :prod -> [host: \"redis.example.com\", port: 6379]\n    end\n  end\nend\n```","ref":"testing_with_cache.html#configuring-your-cache-for-testing"},{"type":"extras","title":"Setting Up the Sandbox Registry - How to Test Applications Using ElixirCache","doc":"To use the sandbox functionality in your tests, you need to start the `Cache.SandboxRegistry` in your test setup:\n\n```elixir\n# In test/test_helper.exs\nExUnit.start()\n\n# Start the sandbox registry for your tests\n{:ok, _pid} = Cache.SandboxRegistry.start_link()\n\n# Start your application's supervision tree\nApplication.ensure_all_started(:my_app)\n```","ref":"testing_with_cache.html#setting-up-the-sandbox-registry"},{"type":"extras","title":"Using the Sandbox in Tests - How to Test Applications Using ElixirCache","doc":"Using the sandbox in your tests is very simple. All you need to do is start the sandbox registry in your setup block:\n\n```elixir\ndefmodule MyApp.CacheTest do\n  use ExUnit.Case, async: true\n\n  defmodule TestCache do\n    use Cache,\n      adapter: Cache.Redis,  # The actual adapter doesn't matter in sandbox mode\n      name: :test_cache,\n      opts: [],\n      sandbox?: Mix.env() === :test\n  end\n\n  setup do\n    # This single line is all you need to set up sandbox isolation\n    Cache.SandboxRegistry.start(TestCache)\n    :ok\n  end\n  \n  test \"can store and retrieve values\" do\n    assert :ok = TestCache.put(\"test-key\", \"test-value\")\n    assert {:ok, \"test-value\"} = TestCache.get(\"test-key\")\n  end\n  \n  test \"can handle complex data structures\" do\n    data = %{users: [%{name: \"Alice\"}, %{name: \"Bob\"}]}\n    assert :ok = TestCache.put(\"complex-key\", data)\n    assert {:ok, ^data} = TestCache.get(\"complex-key\")\n  end\n\n  test \"provides isolation between tests\" do\n    # This will return nil because each test has an isolated cache\n    assert {:ok, nil} = TestCache.get(\"test-key\")\n  end\nend\n```","ref":"testing_with_cache.html#using-the-sandbox-in-tests"},{"type":"extras","title":"Tips for Testing with ElixirCache - How to Test Applications Using ElixirCache","doc":"1. **Always use the sandbox in tests**: This prevents interference between tests.\n2. **Clean up after each test**: Use `on_exit` to unregister from the sandbox.\n3. **Use unique keys**: Even with sandboxing, using descriptive, unique keys makes debugging easier.\n4. **Test edge cases**: Including cache misses, errors, and TTL expiration.\n5. **Consider using fixtures**: For commonly cached data structures.\n6. **Verify telemetry events**: If your application relies on cache metrics.","ref":"testing_with_cache.html#tips-for-testing-with-elixircache"},{"type":"extras","title":"How to Use Metrics and Telemetry with ElixirCache","doc":"# How to Use Metrics and Telemetry with ElixirCache\n\nElixirCache integrates with Elixir's telemetry system to provide observability for your cache operations. This guide explains how to set up and use these metrics in your application.","ref":"using_metrics.html"},{"type":"extras","title":"Understanding ElixirCache Telemetry Events - How to Use Metrics and Telemetry with ElixirCache","doc":"ElixirCache emits the following telemetry events:\n\n| Event Name | Description |\n|------------|-------------|\n| `[:elixir_cache, :cache, :put]` | Emitted when a value is stored in the cache |\n| `[:elixir_cache, :cache, :get]` | Emitted when a value is retrieved from the cache |\n| `[:elixir_cache, :cache, :get, :miss]` | Emitted when a cache lookup results in a miss |\n| `[:elixir_cache, :cache, :delete]` | Emitted when a value is deleted from the cache |\n| `[:elixir_cache, :cache, :put, :error]` | Emitted when an error occurs during a put operation |\n| `[:elixir_cache, :cache, :get, :error]` | Emitted when an error occurs during a get operation |\n| `[:elixir_cache, :cache, :delete, :error]` | Emitted when an error occurs during a delete operation |","ref":"using_metrics.html#understanding-elixircache-telemetry-events"},{"type":"extras","title":"Setting Up Basic Telemetry Handlers - How to Use Metrics and Telemetry with ElixirCache","doc":"To start collecting metrics, you need to attach handlers to these telemetry events:\n\n```elixir\ndefmodule MyApp.CacheMetrics do\n  def setup do\n    # Attach handlers for cache operations\n    :telemetry.attach(\n      \"cache-get-handler\",\n      [:elixir_cache, :cache, :get],\n      &handle_cache_get/4,\n      nil\n    )\n\n    :telemetry.attach(\n      \"cache-miss-handler\",\n      [:elixir_cache, :cache, :get, :miss],\n      &handle_cache_miss/4,\n      nil\n    )\n\n    :telemetry.attach(\n      \"cache-put-handler\",\n      [:elixir_cache, :cache, :put],\n      &handle_cache_put/4,\n      nil\n    )\n\n    :telemetry.attach(\n      \"cache-delete-handler\",\n      [:elixir_cache, :cache, :delete],\n      &handle_cache_delete/4,\n      nil\n    )\n\n    # Attach handlers for error events\n    :telemetry.attach(\n      \"cache-get-error-handler\",\n      [:elixir_cache, :cache, :get, :error],\n      &handle_cache_error/4,\n      nil\n    )\n\n    :telemetry.attach(\n      \"cache-put-error-handler\",\n      [:elixir_cache, :cache, :put, :error],\n      &handle_cache_error/4,\n      nil\n    )\n\n    :telemetry.attach(\n      \"cache-delete-error-handler\",\n      [:elixir_cache, :cache, :delete, :error],\n      &handle_cache_error/4,\n      nil\n    )\n  end\n\n  def handle_cache_get(_event, _measurements, metadata, _config) do\n    IO.puts(\"Cache GET operation for cache: #{metadata.cache_name}\")\n  end\n\n  def handle_cache_miss(_event, measurements, metadata, _config) do\n    IO.puts(\"Cache MISS for cache: #{metadata.cache_name}, count: #{measurements.count}\")\n  end\n\n  def handle_cache_put(_event, _measurements, metadata, _config) do\n    IO.puts(\"Cache PUT operation for cache: #{metadata.cache_name}\")\n  end\n\n  def handle_cache_delete(_event, _measurements, metadata, _config) do\n    IO.puts(\"Cache DELETE operation for cache: #{metadata.cache_name}\")\n  end\n\n  def handle_cache_error(event, measurements, metadata, _config) do\n    [_, _, operation, _] = event\n    IO.puts(\"Cache #{operation} ERROR for cache: #{metadata.cache_name}: #{inspect(metadata.error)}\")\n  end\nend\n```\n\nStart these handlers in your application supervision tree:\n\n```elixir\ndef start(_type, _args) do\n  # Set up telemetry handlers\n  MyApp.CacheMetrics.setup()\n\n  children = [\n    # ... other children\n    MyApp.Cache\n  ]\n\n  opts = [strategy: :one_for_one, name: MyApp.Supervisor]\n  Supervisor.start_link(children, opts)\nend\n```","ref":"using_metrics.html#setting-up-basic-telemetry-handlers"},{"type":"extras","title":"Integration with Telemetry Metrics - How to Use Metrics and Telemetry with ElixirCache","doc":"ElixirCache works well with `telemetry_metrics` for defining and collecting metrics:\n\n```elixir\ndefmodule MyApp.Metrics do\n  import Telemetry.Metrics\n\n  def metrics do\n    [\n      # Counter metrics\n      counter(\"elixir_cache.cache.get.count\", tags: [:cache_name]),\n      counter(\"elixir_cache.cache.get.miss.count\", tags: [:cache_name]),\n      counter(\"elixir_cache.cache.put.count\", tags: [:cache_name]),\n      counter(\"elixir_cache.cache.delete.count\", tags: [:cache_name]),\n      \n      # Error metrics\n      counter(\"elixir_cache.cache.get.error.count\", tags: [:cache_name, :error]),\n      counter(\"elixir_cache.cache.put.error.count\", tags: [:cache_name, :error]),\n      counter(\"elixir_cache.cache.delete.error.count\", tags: [:cache_name, :error]),\n      \n      # Duration metrics (for span events)\n      distribution(\"elixir_cache.cache.get.duration\", unit: {:native, :millisecond}, tags: [:cache_name]),\n      distribution(\"elixir_cache.cache.put.duration\", unit: {:native, :millisecond}, tags: [:cache_name]),\n      distribution(\"elixir_cache.cache.delete.duration\", unit: {:native, :millisecond}, tags: [:cache_name])\n    ]\n  end\nend\n```","ref":"using_metrics.html#integration-with-telemetry-metrics"},{"type":"extras","title":"Integrating with Common Reporting Tools - How to Use Metrics and Telemetry with ElixirCache","doc":"","ref":"using_metrics.html#integrating-with-common-reporting-tools"},{"type":"extras","title":"Prometheus - How to Use Metrics and Telemetry with ElixirCache","doc":"Using `prometheus_telemetry`:\n\n```elixir\ndefmodule MyApp.PrometheusMetrics do\n  def setup do\n    # Define metrics that Prometheus will collect\n    metrics = MyApp.Metrics.metrics()\n    \n    # Attach Prometheus reporters to these metrics\n    :prometheus_telemetry.setup(metrics)\n    \n    # Start the Prometheus metrics HTTP endpoint\n    Plug.Cowboy.http(PrometheusTelemetry.Plug, [], port: 9568)\n  end\nend\n```","ref":"using_metrics.html#prometheus"},{"type":"extras","title":"StatsD - How to Use Metrics and Telemetry with ElixirCache","doc":"Using `telemetry_metrics_statsd`:\n\n```elixir\ndefmodule MyApp.Application do\n  use Application\n\n  def start(_type, _args) do\n    children = [\n      # Start the Telemetry supervisor\n      {TelemetryMetricsStatsd, metrics: MyApp.Metrics.metrics()},\n      \n      # ... other children\n      MyApp.Cache\n    ]\n\n    opts = [strategy: :one_for_one, name: MyApp.Supervisor]\n    Supervisor.start_link(children, opts)\n  end\nend\n```","ref":"using_metrics.html#statsd"},{"type":"extras","title":"Creating Custom Cache Metrics - How to Use Metrics and Telemetry with ElixirCache","doc":"You can also create custom metrics specific to your application's needs:\n\n```elixir\ndefmodule MyApp.CustomCacheMetrics do\n  # Track the size of values being stored\n  def track_value_size(key, value) do\n    byte_size = :erlang.term_to_binary(value) |> byte_size()\n    \n    :telemetry.execute(\n      [:my_app, :cache, :value_size],\n      %{bytes: byte_size},\n      %{key: key}\n    )\n  end\n  \n  # Track cache hit ratio\n  def track_hit_ratio() do\n    # Attach to both get and miss events\n    :telemetry.attach(\n      \"hit-ratio-get-handler\",\n      [:elixir_cache, :cache, :get],\n      &handle_get/4,\n      %{hits: 0, misses: 0}\n    )\n    \n    :telemetry.attach(\n      \"hit-ratio-miss-handler\",\n      [:elixir_cache, :cache, :get, :miss],\n      &handle_miss/4,\n      %{hits: 0, misses: 0}\n    )\n  end\n  \n  defp handle_get(_event, _measurements, _metadata, config) do\n    hits = config.hits + 1\n    ratio = hits / (hits + config.misses)\n    \n    :telemetry.execute(\n      [:my_app, :cache, :hit_ratio],\n      %{ratio: ratio},\n      %{}\n    )\n    \n    %{hits: hits, misses: config.misses}\n  end\n  \n  defp handle_miss(_event, _measurements, _metadata, config) do\n    misses = config.misses + 1\n    ratio = config.hits / (config.hits + misses)\n    \n    :telemetry.execute(\n      [:my_app, :cache, :hit_ratio],\n      %{ratio: ratio},\n      %{}\n    )\n    \n    %{hits: config.hits, misses: misses}\n  end\nend\n```","ref":"using_metrics.html#creating-custom-cache-metrics"},{"type":"extras","title":"Best Practices for Cache Metrics - How to Use Metrics and Telemetry with ElixirCache","doc":"1. **Monitor hit/miss ratio**: A low hit ratio may indicate cache keys that expire too quickly or ineffective caching strategies.\n\n2. **Track cache size**: For in-memory caches like ETS, monitor the memory usage to prevent excessive memory consumption.\n\n3. **Watch for error rates**: Sudden increases in error events might indicate connectivity issues (for Redis) or other problems.\n\n4. **Measure operation durations**: Unusually long cache operations could signal network latency or overloaded cache servers.\n\n5. **Set alerts**: Configure alerts for abnormal metrics like high miss rates, increasing error rates, or excessive latency.\n\n6. **Segment by cache name**: Always include the cache name in your metrics to distinguish between different caches in your application.\n\n7. **Correlate with application metrics**: Analyze how cache performance affects overall application performance.","ref":"using_metrics.html#best-practices-for-cache-metrics"},{"type":"extras","title":"Architecture and Core Concepts","doc":"# Architecture and Core Concepts\n\nThis document explains the key architectural concepts behind ElixirCache and how its components work together.","ref":"architecture.html"},{"type":"extras","title":"Core Architecture - Architecture and Core Concepts","doc":"ElixirCache is designed around a simple principle: provide a consistent interface to different caching backends. The architecture consists of:\n\n1. **Core Interface**: Defined by the `Cache` module\n2. **Adapters**: Backend-specific implementations\n3. **Term Encoder**: Handles serialization and compression\n4. **Telemetry Integration**: For observability and metrics\n5. **Sandbox System**: For isolated testing","ref":"architecture.html#core-architecture"},{"type":"extras","title":"The Cache Behaviour - Architecture and Core Concepts","doc":"At the heart of ElixirCache is the `Cache` behaviour, which defines the contract that all cache adapters must implement:\n\n- `child_spec/1`: Defines how the cache is started and supervised\n- `opts_definition/0`: Defines adapter-specific options\n- `put/5`: Stores a value in the cache\n- `get/3`: Retrieves a value from the cache\n- `delete/3`: Removes a value from the cache\n\nEach adapter implements these functions, allowing your application code to remain the same regardless of which cache backend you use.","ref":"architecture.html#the-cache-behaviour"},{"type":"extras","title":"The `use Cache` Macro - Architecture and Core Concepts","doc":"When you `use Cache` in your module, the macro:\n\n1. Sets up the configuration for your cache\n2. Creates the required module functions that delegate to the appropriate adapter\n3. Wraps operations in telemetry spans for metrics and observability\n4. Implements the `get_or_create/2` convenience function\n5. Adds sandboxing capabilities for testing if enabled","ref":"architecture.html#the-use-cache-macro"},{"type":"extras","title":"Term Encoding and Compression - Architecture and Core Concepts","doc":"ElixirCache includes internal term encoding functionality that handles serialization and deserialization of Elixir terms. This allows you to store complex Elixir data structures in any cache backend. The encoding system:\n\n1. Uses Erlang's term_to_binary for efficient serialization\n2. Applies configurable compression to reduce memory usage\n3. Automatically handles decoding when retrieving values","ref":"architecture.html#term-encoding-and-compression"},{"type":"extras","title":"Sandboxing for Tests - Architecture and Core Concepts","doc":"A unique feature of ElixirCache is its sandboxing capability for tests. When you enable sandboxing:\n\n1. Each test has its own isolated cache namespace\n2. Cache operations are automatically prefixed with a test-specific ID\n3. Tests can run concurrently without cache interference or global overlap\n\nThis is achieved through the `Cache.SandboxRegistry` which maintains a registry of cache contexts.","ref":"architecture.html#sandboxing-for-tests"},{"type":"extras","title":"Adapters Design - Architecture and Core Concepts","doc":"Each adapter is designed to be a thin wrapper around the underlying storage mechanism:","ref":"architecture.html#adapters-design"},{"type":"extras","title":"ETS Adapter - Architecture and Core Concepts","doc":"The ETS adapter uses Erlang Term Storage for high-performance in-memory caching. It:\n- Starts an ETS table with the configured settings\n- Provides direct access to ETS-specific functions\n- Handles conversion between the Cache interface and ETS operations","ref":"architecture.html#ets-adapter"},{"type":"extras","title":"DETS Adapter - Architecture and Core Concepts","doc":"Similar to the ETS adapter but uses disk-based storage for persistence across restarts.","ref":"architecture.html#dets-adapter"},{"type":"extras","title":"Redis Adapter - Architecture and Core Concepts","doc":"The Redis adapter provides a more feature-rich distributed caching solution:\n- Manages a connection pool to Redis\n- Handles serialization of Elixir terms for Redis storage\n- Provides access to Redis-specific operations like hash and JSON commands","ref":"architecture.html#redis-adapter"},{"type":"extras","title":"Agent Adapter - Architecture and Core Concepts","doc":"A simple implementation using Elixir's Agent for lightweight in-memory storage.","ref":"architecture.html#agent-adapter"},{"type":"extras","title":"ConCache Adapter - Architecture and Core Concepts","doc":"Wraps the ConCache library to provide its expiration and callback capabilities.","ref":"architecture.html#concache-adapter"},{"type":"extras","title":"Telemetry Integration - Architecture and Core Concepts","doc":"ElixirCache provides telemetry events for all cache operations:\n- `[:elixir_cache, :cache, :put]` - When storing values\n- `[:elixir_cache, :cache, :get]` - When retrieving values\n- `[:elixir_cache, :cache, :get, :miss]` - When a key is not found\n- `[:elixir_cache, :cache, :delete]` - When deleting values\n- Error events when operations fail\n\nThis allows you to monitor cache performance, hit/miss ratios, and error rates.","ref":"architecture.html#telemetry-integration"},{"type":"extras","title":"API Reference","doc":"# API Reference\n\nThis document provides a comprehensive overview of the ElixirCache API and all available functions.","ref":"api.html"},{"type":"extras","title":"Core Cache API - API Reference","doc":"These functions are available on any cache module that uses the `Cache` behavior.","ref":"api.html#core-cache-api"},{"type":"extras","title":"put/3 - API Reference","doc":"```elixir\nput(key, value)\nput(key, ttl, value)\n```\n\nStores a value in the cache.\n\n- `key` - The key under which to store the value (atom or string)\n- `ttl` - Optional time-to-live in seconds (integer)\n- `value` - The value to store (any Elixir term)\n\nReturns `:ok` or `{:error, reason}`.","ref":"api.html#put-3"},{"type":"extras","title":"get/1 - API Reference","doc":"```elixir\nget(key)\n```\n\nRetrieves a value from the cache.\n\n- `key` - The key to look up (atom or string)\n\nReturns `{:ok, value}`, `{:ok, nil}` if the key doesn't exist, or `{:error, reason}`.","ref":"api.html#get-1"},{"type":"extras","title":"delete/1 - API Reference","doc":"```elixir\ndelete(key)\n```\n\nRemoves a value from the cache.\n\n- `key` - The key to delete (atom or string)\n\nReturns `:ok` or `{:error, reason}`.","ref":"api.html#delete-1"},{"type":"extras","title":"get_or_create/2 - API Reference","doc":"```elixir\nget_or_create(key, function)\n```\n\nRetrieves a value from the cache if it exists, or executes the provided function to create and store it.\n\n- `key` - The key to look up or create (atom or string)\n- `function` - A function that returns `{:ok, value}` or `{:error, reason}`\n\nReturns `{:ok, value}` or `{:error, reason}`.","ref":"api.html#get_or_create-2"},{"type":"extras","title":"Adapter-Specific Functions - API Reference","doc":"","ref":"api.html#adapter-specific-functions"},{"type":"extras","title":"ETS Adapter - API Reference","doc":"In addition to the core cache functions, ETS cache modules provide:\n\n```elixir\nmatch_object(pattern)\nmatch_object(pattern, limit)\nmember(key)\nselect(match_spec)\nselect(match_spec, limit)\ninfo()\ninfo(item)\nselect_delete(match_spec)\nmatch_delete(pattern)\nupdate_counter(key, update_op)\ninsert_raw(data)\n```","ref":"api.html#ets-adapter"},{"type":"extras","title":"DETS Adapter - API Reference","doc":"DETS cache modules provide the same functions as ETS, plus:\n\n```elixir\nto_ets(ets_table)\nfrom_ets(ets_table)\n```","ref":"api.html#dets-adapter"},{"type":"extras","title":"Redis Adapter - API Reference","doc":"Caches configured with the Redis adapter provide additional functionality beyond the standard cache operations. When you define a cache module using the Redis adapter, these functions become available on your cache module.\n\n#### Additional Redis Operations\n\nBelow are the Redis-specific operations that your cache module will have available:\n\n```elixir\n# Define a cache module with the Redis adapter\ndefmodule MyApp.RedisCache do\n  use Cache,\n    adapter: Cache.Redis,\n    name: :my_app_redis_cache,\n    opts: [uri: \"redis://localhost:6379\"]\nend\n\n# Hash operations\nMyApp.RedisCache.hash_get(\"user:1\", \"name\")                  # Get a hash field\nMyApp.RedisCache.hash_get_all(\"user:1\")                     # Get all fields in a hash\nMyApp.RedisCache.hash_set(\"user:1\", \"email\", \"user@example.com\") # Set a hash field\nMyApp.RedisCache.hash_delete(\"user:1\", \"temporary_field\")   # Delete a hash field\n\n# JSON operations\nMyApp.RedisCache.json_get(\"config\")                         # Get entire JSON\nMyApp.RedisCache.json_set(\"config\", \".settings\", %{theme: \"dark\"}) # Set JSON path\nMyApp.RedisCache.json_incr(\"stats\", \".counter\")             # Increment value at path\n\n# Direct Redis access\nMyApp.RedisCache.command([\"PING\"])                          # Run a Redis command\nMyApp.RedisCache.pipeline([[\"SET\", \"key1\", \"val1\"], [\"GET\", \"key2\"]]) # Run commands in pipeline\n```\n\n#### Full List of Available Redis Functions\n\nYour Redis cache module will include these functions:\n\n* **Hash Functions**: `hash_get/2`, `hash_get_all/1`, `hash_get_many/1`, `hash_values/1`, `hash_set/3`, `hash_set/4`, `hash_set_many/2`, `hash_delete/2`, `hash_scan/1`, `hash_scan/2`\n\n* **JSON Functions**: `json_get/1`, `json_get/2`, `json_set/3`, `json_delete/2`, `json_incr/2`, `json_incr/3`, `json_array_append/3`\n\n* **Command Functions**: `command/1`, `command/2`, `command!/1`, `command!/2`, `pipeline/1`, `pipeline/2`, `pipeline!/1`, `pipeline!/2`, `scan/0`, `scan/1`\n\n* **Set Functions**: `smembers/2`, `sadd/2`, `sadd/3`","ref":"api.html#redis-adapter"},{"type":"extras","title":"Telemetry Events - API Reference","doc":"ElixirCache emits the following telemetry events:\n\n| Event Name | Measurements | Metadata |\n|------------|--------------|----------|\n| `[:elixir_cache, :cache, :put]` | `%{}` | `%{cache_name: atom}` |\n| `[:elixir_cache, :cache, :put, :error]` | `%{count: 1}` | `%{cache_name: atom, error: term}` |\n| `[:elixir_cache, :cache, :get]` | `%{}` | `%{cache_name: atom}` |\n| `[:elixir_cache, :cache, :get, :miss]` | `%{count: 1}` | `%{cache_name: atom}` |\n| `[:elixir_cache, :cache, :get, :error]` | `%{count: 1}` | `%{cache_name: atom, error: term}` |\n| `[:elixir_cache, :cache, :delete]` | `%{}` | `%{cache_name: atom}` |\n| `[:elixir_cache, :cache, :delete, :error]` | `%{count: 1}` | `%{cache_name: atom, error: term}` |","ref":"api.html#telemetry-events"},{"type":"extras","title":"0.3.10","doc":"# 0.3.10\n- fix: sandbox fix for smembers & sadd\n\n# 0.3.9\n- chore: add docs\n- fix: set fix\n\n# 0.3.8\n- feat: add functions for ets & dets caches\n\n# 0.3.7\n- feat: add metrics module\n\n# 0.3.6\n- chore: fix child_spec type\n\n# 0.3.5\n- fix: Cache child spec for starting under a supervisor\n\n# 0.3.4\n- add `get_or_create(key, (() -> {:ok, value} | {:error, reson}))` to allow for create or updates\n\n# 0.3.3\n- use adapter options to allow for runtime options\n- update sandbox hash_set_many behaviour to be consistent\n- ensure dets does a mkdir_p at startup incase directory doesn't exist\n\n# 0.3.2\n- Update nimble options to 1.x\n\n# 0.3.1\n- add some more json sandboxing\n- update redis to remove uri from command options\n\n# 0.3.0\n- add con_cache\n- add ets cache\n- fix hash opts for redis\n\n# 0.2.1\n- Adds support for application configuration and runtime options\n\n# 0.2.0\n- Stop redis connection errors from crashing the app\n- Fix hash functions for `Cache.Redis`\n- Support runtime cache config\n- Support redis JSON\n- Add `strategy` option to `Cache.Redis` for poolboy\n\n# 0.1.1\n- Expose `pipeline` and `command` functions on redis adapters\n\n# 0.1.0\n- Initial Release","ref":"changelog.html"}],"proglang":"elixir","content_type":"text/markdown","producer":{"name":"ex_doc","version":"0.37.3"}}